/* EMX-specific functions
 */

#if !defined(lint) && !defined(LINT)
static char rcsid[] = "$Id: os2env.c,v 1.1.2.2 2001/03/07 22:17:39 root Exp $";
#endif


#include <stdlib.h>
#include <stdarg.h>
#define INCL_BASE
#include <os2emx.h>

#include "cron.h"

static void error(char *fmt, ...)
{
 va_list vf;

 va_start(vf, fmt);
 vprintf(fmt, vf);
 va_end(vf);
 exit(1);
}

static void *malloc_msg(int s)
{
 void *rc;

 if((rc=malloc(s))==NULL)
  error("Failed to allocate %d bytes\n", s);
 return(rc);
}

char *igetenv(char *s, char *def)
{
 char *rc;

 if((rc=getenv(s))==NULL)
  return(def);
 else
  return(rc);
}

static char *cgetenv(char *s)
{
 char *rc;

 if((rc=getenv(s))==NULL)
  error("Error (EMX-specific): Environment variable \"%s\" not set.\n", s);
 return(rc);
}

static void fixpath(char *s)
{
 char *p;
 int l;

 if((p=strchr(s, ';'))!=NULL)
  *p='\0';
 l=strlen(s);
 if(l>0&&s[l-1]!='\\'&&s[l-1]!='/'&&s[l-1]!=':')
  strcat(s, (strchr(s, '/')==NULL)?"\\":"/");
}

char *getenv_safe(char *s)
{
 char *rc;
 static char *s_crondir=NULL;
 static char *s_cronddir=NULL;
 static char *s_syscrontab=NULL;

 if(!stricmp(s, "TMPDIR"))
  return(igetenv(s, cgetenv("TMP")));
 if(!stricmp(s, "CRONDIR"))
 {
  if((rc=getenv(s))!=NULL)
   return(rc);
  else if(s_crondir!=NULL)
   return(s_crondir);
  else
  {
   s_crondir=malloc_msg(CCHMAXPATH);
   strcpy(s_crondir, cgetenv("ETC"));
   fixpath(s_crondir);
   strcat(s_crondir, "cron");           /* /etc/cron */
  }
 }
 else if(!stricmp(s, "CRONDDIR"))
 {
  if((rc=getenv(s))!=NULL)
   return(rc);
  else if(s_cronddir!=NULL)
   return(s_cronddir);
  else
  {
   s_cronddir=malloc_msg(CCHMAXPATH);
   strcpy(s_cronddir, cgetenv("ETC"));
   fixpath(s_cronddir);
   strcat(s_cronddir, "cron.d");        /* /etc/cron.d */
  }
 }
 else if(!stricmp(s, "PIDDIR"))
  return(igetenv(s, getenv_safe("TMPDIR")));
 else if(!stricmp(s, "SYSCRONTAB"))
 {
  if((rc=getenv(s))!=NULL)
   return(rc);
  else if(s_syscrontab!=NULL)
   return(s_syscrontab);
  else
  {
   s_syscrontab=malloc_msg(CCHMAXPATH);
   strcpy(s_syscrontab, cgetenv("ETC"));
   fixpath(s_syscrontab);
   strcat(s_syscrontab, "crontab");        /* /etc/crontab */
  }
 }
 else if(!stricmp(s, "EDITOR"))
  return(igetenv(s, "epm"));            /* E doesn't work */
 else if(!stricmp(s, "_PATH_BSHELL"))
  return(igetenv(s, "sh"));
 else if(!stricmp(s, "_PATH_DEFPATH"))
  return(igetenv(s, "PATH"));
 else
  return(cgetenv(s));
}
